import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

/**
 * @author <A HREF="mailto:weyer@kagi.com">Steve Weyer</A>
 * Copyright (c) 2001, S. Weyer. All right reserved.
 * @version Personal Java 1.0
 * <A HREF="http://www.kagi.com/weyer/">Steve's other apps</A>
 *
 */

public class CountDown
  extends Frame
  implements
    Runnable,
    ItemListener     // for Choice
  {
  private static final int kSecMs = 1000;
  private static final int kMinMs = 60 * kSecMs;
  private static final int kHrMs  = 60 * kMinMs;
  private static final int kDayMs = 24 * kHrMs;
  private static final int kWeekMs = 7 * kDayMs;
  private static final int kYearMs = 365 * kDayMs;
  private static final String undate = "???w ??d ??h ??m ??s";

  // north
  private Label nowVal;

  // center
  private Choice inputChoices;
  private TextField newInput;

  // south
  private Label waitVal;

  private SimpleDateFormat dfi = new SimpleDateFormat("d MMM yyyy"); //SimpleDateFormat.getDateTimeInstance();
  private SimpleDateFormat dfo = new SimpleDateFormat("d MMM yyyy h:mm:ss a"); //SimpleDateFormat.getDateTimeInstance();

  private Thread clockThread;
  private Date setDate;
  private Random rand;

  private Date [] choiceDates = {
    myGetDate("29 Sep 2002"),
    myGetDate("25 Dec 2001"),
    myGetDate("1 Jan 2002"),
  };
  private String [] choiceNames = { // coord with choiceDates
    "Ellie's Birthday '02",
    "Christmas '01",
    "New Year's 2002",
  };

  public static void main(String[] args) {
    new CountDown();
  }

  public CountDown() {
    super("Ellie's CountDown");
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent ev) { System.exit(0); } });
    setLayout(new BorderLayout());

    // north: <nowLabel> <nowVal>
    Panel p = new Panel();
    p.add(new Label("Now:"));
    p.add(nowVal = new Label(dfo.format(new Date())));
    add(p, BorderLayout.NORTH);

    // center: <inputChoices> <newInput>
    p = new Panel();
    inputChoices = new Choice();
    inputChoices.addItem("(Use Field)");
    for (int i=0; i < choiceNames.length; i++)
      inputChoices.addItem(choiceNames[i]);
    inputChoices.addItem("End of World?");
    inputChoices.addItemListener(this);
    p.add(inputChoices);

    newInput = new TextField(15); // 35
    p.add(newInput);
    add(p, BorderLayout.CENTER);

    // south: <diffLabel> <waitVal>
    p = new Panel();
    p.add(new Label("You'll have to wait:"));
    p.add(waitVal = new Label(undate));
    add(p, BorderLayout.SOUTH);

    //pack();
    Dimension screenSize = getToolkit().getScreenSize();
    setSize(Math.min(screenSize.width,280), Math.min(screenSize.height,140));
    setVisible(true);

    rand = new Random();
    clockThread = new Thread(this);
    clockThread.start();
  }

  private Date myGetDate(String s) {
    try {
      return dfi.parse(s);
    }
    catch (ParseException ex) {
      //System.err.println("myGetDate err: " + s);
      return null;
    };
  }

  public int nextInt(int n) { // >= 1.2 rand.nextInt(10));
    return (int) (rand.nextFloat() * n);
  }

  public void getChoice() { // share itemStateChanged, action
    int item = inputChoices.getSelectedIndex();
    if (item == 0)
      // convert user input
      setDate = myGetDate(newInput.getText());
    else if (item <= choiceDates.length)
      // existing menu date
      setDate = choiceDates[item-1];
    else {
      // random date in next 10+ years
      GregorianCalendar cal = new GregorianCalendar();
      cal.add(Calendar.YEAR,        nextInt(10));
      cal.add(Calendar.MONTH,       nextInt(12));
      cal.add(Calendar.DAY_OF_MONTH,nextInt(30));
      cal.add(Calendar.HOUR,        nextInt(24));
      cal.add(Calendar.MINUTE,      nextInt(60));
      cal.add(Calendar.SECOND,      nextInt(60));
      setDate = cal.getTime();
    };
    //System.out.println(setDate);

    if (setDate != null)
      newInput.setText(dfi.format(setDate));
  }

  public void itemStateChanged(ItemEvent ev) {
    //if (ev.getSource() == inputChoices)
      getChoice();
  }

  public void updateDate() {
    Date now = new Date();							// 1.0
    nowVal.setText(dfo.format(now));

    if (setDate == null)
      waitVal.setText(undate);
    else {
      long newMs = setDate.getTime();
      long nowMs = now.getTime();
      long diff = newMs - nowMs;
      int wks = (int) (diff / kWeekMs);
      diff = diff % kWeekMs;
      int days = (int) (diff / kDayMs);
      diff = diff % kDayMs;
      int hrs = (int) (diff / kHrMs);
      diff = diff % kHrMs;
      int mins = (int) (diff / kMinMs);
      diff = diff % kMinMs;
      int secs = (int) Math.ceil((double) diff / kSecMs);
      waitVal.setText(
        wks + "w " + days + "d " + hrs + "h " +
        mins + "m " + secs + 's');
      };
  }

  public void run() { // via clockThread & Runnable
    while (true) {  // ??? stop condition other than exit
      try {
        updateDate();
        clockThread.sleep(1000);
      }
      catch (InterruptedException ex) {
        System.err.println("thread: " + ex);
      };
    }
  }
}
